const { getJwks } = require('./jwks.js');
const bodyParser = require('body-parser');
let configuration = {};
let app; // This will hold an express app instance.

/*
    Example Usage:
    const express = require('express');
    const lti = require('canvas-lti');
    const app = express();

    lti.expressInstance(app);
    lti.on('/lti/launch', async (req, res) => {
        // Your code here
    });
*/

const config = (options) => {
    const requiredOptions = [
        'oidcInitiationUrl',
        'clientId',
        'baseURL',
        'launchPath',
        'ltiPath'
    ]
    const missingOptions = requiredOptions.filter(option => !(option in options));
    if (missingOptions.length > 0) {
        throw new Error(`Missing required options: ${missingOptions.join(', ')}`);
    }
    configuration = {...configuration, ...options};
}

const lti =  {
    config,
    expressInstance: function(expressInstance) {
        app = expressInstance;
        app.use(bodyParser.urlencoded({ extended: true }));

        const jwksPath = configuration.ltiPath ? `${configuration.ltiPath}/.well-known/jwks.json` : '/.well-known/jwks.json';
        app.get(jwksPath, async (req, res) => {
            try {
                const jwks = await getJwks();
                res.status(200).json(jwks);
            } catch (err) {
                console.error(err);
                res.status(500).json({ error: 'Unable to retrieve the JWKS for this application.' });
            }
        });

        const oidcPath = configuration.ltiPath ? `${configuration.ltiPath}/oidc` : '/oidc';
        app.post(oidcPath, async (req, res) => {
            const { login_hint, lti_message_hint } = req.body;
            const oidcConfigUrl = configuration.oidcInitiationUrl;
            const redirectUri = configuration.baseURL + configuration.launchPath;
            const CLIENT_ID = configuration.clientId;

            const oidcReq = {
                client_id: CLIENT_ID,
                login_hint: login_hint,
                lti_message_hint: lti_message_hint,
                nonce: crypto.randomUUID(),
                prompt: 'none',
                redirect_uri: redirectUri,
                response_mode: 'form_post',
                response_type: 'id_token',
                scope: 'openid',
                state: crypto.randomUUID()
            };

            const queryString = Object.keys(oidcReq)
                .map(key => `${key}=${encodeURIComponent(oidcReq[key])}`)
                .join('&');
            
            const fullUrl = `${oidcConfigUrl}?${queryString}`;
            res.redirect(fullUrl);
        });
    },
    on: (path, handler) => {
        if (!configuration.ltiPath.startsWith('/')) configuration.ltiPath = `/${configuration.ltiPath}`;
        if (!configuration.ltiPath.endsWith('/')) configuration.ltiPath = `${configuration.ltiPath}/`;
        
        const fullPath = configuration.ltiPath ? `${configuration.ltiPath}${path}` : path;

        app.post(fullPath, async (req, res) => {
            try {
                await handler(req, res);
            } catch (err) {
                console.error(err);
                res.status(500).send('Error');
            }
        });
    }
}

module.exports = lti;